function [map_walls ] = map_def()
% MAP_DEF Create a map where the walls and the obstacles are represented as
% lines.
%
% OUTPUT:
% - map_walls: a matrix containing the line segments 
%
% The i-th segment, AB, is defined as a cell s{i}=[xa ya; xb yb]'.
%
% Insert all the segments

% Insert the semgents of the map 

% s{1} = [0     15;     15      15]';
% s{2} = [15    15;     15      3.5]';
% s{3} = [15    1.5;    15      0]';
% s{4} = [15    0;      0       0]';
% s{5} = [0     0;      0       15]';
% s{6} = [15    0;      25      0]';
% s{7} = [15    15;     25      15]';
% s{8} = [25    15;     25      13.5]';
% s{9} = [25    11.5;   25      0]';
% s{10}= [25    8;      30      8]';
% s{11}= [30    8;      30      15]';
% s{12}= [30    15;     28.5    15]';
% s{13}= [25    15;     27      15]';
% s{14}= [26    15;     26      23]';
% s{15}= [29    15;     29      23]';
% s{16}= [26    23;     26.5    23]';
% s{17}= [29    23;     28.5    23]';

% s{1}=[0 0; 0 14]';
% s{2}=[7 0; 7 10]';
% s{3}=[0 7.5; 2.5 7.5]';
% s{4}=[4.5 7.5; 7 7.5]';
% s{5}=[0 14; 11 14]';
% s{6}=[7 9; 14 9]';
% s{7}=[7 12; 7 14]';
% s{8}=[14 9; 14 14]';
% s{9}=[13 14; 14 14]';
% s{10}=[11 14; 11 19]';
% s{11}=[13 14; 13 19]';

offsy=3; %y offset from y=0
offsx=0.2; %x offset for obstacles
len=10; %length of corridor
wid=3; %width of corridor

%%% static environment %%%
%left shelve
s{1}=[0 offsy; 1 offsy]';
s{2}=[1 offsy; 1 len+offsy]';
s{3}=[1 len+offsy; 0 len+offsy]';

%right shelve
s{4}=[1+wid offsy; 2+wid offsy]';
s{5}=[1+wid offsy; 1+wid len+offsy]';
s{6}=[1+wid len+offsy; 2+wid len+offsy]';

%static obstacles
osize1=0.4; %size parameters for obstacles
osize2=0.2;
osize3=0.1;
midy=offsy+len/2;

%obstacle 1
s{7}=[1+offsx midy;1+offsx+osize1 midy]';
s{8}=[1+offsx+osize1 midy;1+offsx+osize1 midy-osize1]';
s{9}=[1+offsx+osize1 midy-osize1;1+offsx midy-osize1]';
s{10}=[1+offsx midy-osize1; 1+offsx midy]';

%obstacle 2
s{11}=[1+wid-offsx 7;1+wid-offsx-osize2 7]';
s{12}=[1+wid-offsx-osize2 7;1+wid-offsx-osize2 7-osize2]';
s{13}=[1+wid-offsx-osize2 7-osize2;1+wid-offsx 7-osize2]';
s{14}=[1+wid-offsx 7-osize2; 1+wid-offsx 7]';

%obstacle 3
s{15}=[1+wid-offsx offsy+1;1+wid-offsx-osize1 offsy+1]';
s{16}=[1+wid-offsx-osize1 offsy+1;1+wid-offsx-osize1 offsy+1-osize3]';
s{17}=[1+wid-offsx-osize1 offsy+1-osize3;1+wid-offsx offsy+1-osize3]';
s{18}=[1+wid-offsx offsy+1-osize3; 1+wid-offsx offsy+1]';

%obstacle 4
s{19}=[1+wid-offsx*2 10;1+wid-offsx*2-osize3 10]';
s{20}=[1+wid-offsx*2-osize3 10;1+wid-offsx*2-osize3 10-osize1]';
s{21}=[1+wid-offsx*2-osize3 10-osize1;1+wid-offsx*2 10-osize1]';
s{22}=[1+wid-offsx*2 10-osize1; 1+wid-offsx*2 10]';

%shopping carts
dim=[0.6 0.5 1]; %dimensions of shoppingcart

%shopping cart 1

s{23}=[1+offsx offsy+len-1; 1+offsx-(dim(1)-dim(2)) offsy+len-1-dim(3)]';
s{24}=[1+offsx-(dim(1)-dim(2)) offsy+len-1-dim(3);1+offsx+dim(2) offsy+len-1-dim(3)]';
s{25}=[1+offsx+dim(2) offsy+len-1-dim(3);1+offsx+dim(2)-(dim(1)-dim(2)) offsy+len-1]';
s{26}=[1+offsx+dim(2)-(dim(1)-dim(2)) offsy+len-1; 1+offsx offsy+len-1]';

%shopping cart 2

s{27}=[1+offsx offsy+2; 1+offsx-(dim(1)-dim(2)) offsy+2-dim(3)]';
s{28}=[1+offsx-(dim(1)-dim(2)) offsy+2-dim(3);1+offsx+dim(2) offsy+2-dim(3)]';
s{29}=[1+offsx+dim(2) offsy+2-dim(3);1+offsx+dim(2)-(dim(1)-dim(2)) offsy+2]';
s{30}=[1+offsx+dim(2)-(dim(1)-dim(2)) offsy+2; 1+offsx offsy+2]';

%shopping cart 3

s{31}=[1+offsx offsy+4; 1+offsx-(dim(1)-dim(2)) offsy+4-dim(3)]';
s{32}=[1+offsx-(dim(1)-dim(2)) offsy+4-dim(3);1+offsx+dim(2) offsy+4-dim(3)]';
s{33}=[1+offsx+dim(2) offsy+4-dim(3);1+offsx+dim(2)-(dim(1)-dim(2)) offsy+4]';
s{34}=[1+offsx+dim(2)-(dim(1)-dim(2)) offsy+4; 1+offsx offsy+4]';

n_s=length(s); % Number of segments

map_walls=[];
for i=1:n_s
    map_walls=[map_walls;s{i}];
end
        
end

